searchData={"content_type":"text/markdown","items":[{"doc":"Authentication & sessions for API's.","ref":"Charon.html","title":"Charon","type":"module"},{"doc":"Config struct.\n\nAll config is read at runtime. So if, for example, you wish to override `:session_ttl`\nbased on the username, you can simply alter the config struct in your code.\n\nThat being said, config that HAS to be read at runtime, like secrets,\nis stored as a getter to emphasize the fact and prevent you from accidentally setting\na compile-time value even if you put the config struct in a module attribute.\nThat is why the base secret has to be passed in via `:get_base_secret`.","ref":"Charon.Config.html","title":"Charon.Config","type":"module"},{"doc":"[\n      :token_issuer,\n      :get_base_secret,\n      access_cookie_name: \"_access_token_signature\",\n      access_cookie_opts: [http_only: true, same_site: \"Strict\", secure: true],\n      # 15 minutes\n      access_token_ttl: 15 * 60,\n      enforce_browser_cookies: false,\n      json_module: Jason,\n      optional_modules: %{},\n      refresh_cookie_name: \"_refresh_token_signature\",\n      refresh_cookie_opts: [http_only: true, same_site: \"Strict\", secure: true],\n      # 2 months\n      refresh_token_ttl: 2 * 30 * 24 * 60 * 60,\n      session_store_module: Charon.SessionStore.RedisStore,\n      # 1 year\n      session_ttl: 365 * 24 * 60 * 60,\n      token_factory_module: Charon.TokenFactory.Jwt\n    ]","ref":"Charon.Config.html#module-keys-defaults","title":"Keys & defaults - Charon.Config","type":"module"},{"doc":"- `:access_cookie_name` Name of the cookie in which the access token or its signature is stored.\n - `:access_cookie_opts` Options passed to `Plug.Conn.put_resp_cookie/3`. Note that `:max_age` is set by `Charon.SessionPlugs` based on the token TTL. Overrides are merged into the defaults.\n - `:access_token_ttl` Time in seconds until a new access token expires. This time may be reduced so that the token does not outlive its session.\n - `:enforce_browser_cookies` If a browser client is detected, enforce that tokens are not returned to it as fully valid bearer tokens, but are transported (wholly or in part) as cookies.\n - `:get_base_secret` Getter for Charon's base secret from which other keys are derived. Make sure it has large entropy (>= 256 bits). For example `fn -> Application.get_env(:my_app, :charon_secret) end`.\n - `:json_module` The JSON module, like `Jason` or `Poison`.\n - `:optional_modules` Configuration for optional modules, like `Charon.TokenFactory.Jwt` or `CharonOauth2`. See the optional module's docs for info on its configuration options.\n - `:refresh_cookie_name` Name of the cookie in which the refresh token or its signature is stored.\n - `:refresh_cookie_opts` Options passed to `Plug.Conn.put_resp_cookie/3`. Note that `:max_age` is set by `Charon.SessionPlugs` based on the token TTL. Overrides are merged into the defaults.\n - `:refresh_token_ttl` Time in seconds until a new refresh token expires. This time may be reduced so that the token does not outlive its session.\n - `:session_store_module` A module that implements `Charon.SessionStore.Behaviour`, used to store sessions.\n - `:session_ttl` Time in seconds until a new session expires OR `:infinite` for non-expiring sessions.\n - `:token_factory_module` A module that implements `Charon.TokenFactory.Behaviour`, used to create and verify authentication tokens.\n - `:token_issuer` Value of the \"iss\" claim in tokens, for example \"https://myapp.com\"","ref":"Charon.Config.html#module-glossary","title":"Glossary - Charon.Config","type":"module"},{"doc":"Build config struct from enumerable (useful for passing in application environment).\nRaises for missing mandatory keys and sets defaults for optional keys.\nOptional modules must implement an `init_config/1` function to process their own config at compile time.","ref":"Charon.Config.html#from_enum/1","title":"Charon.Config.from_enum/1","type":"function"},{"doc":"iex> from_enum([])\n    ** (ArgumentError) the following keys must also be given when building struct Charon.Config: [:token_issuer, :get_base_secret]\n\n    iex> %Charon.Config{} = from_enum(token_issuer: \"https://myapp\", get_base_secret: \"supersecure\")","ref":"Charon.Config.html#from_enum/1-examples-doctests","title":"Examples / doctests - Charon.Config.from_enum/1","type":"function"},{"doc":"","ref":"Charon.Config.html#t:t/0","title":"Charon.Config.t/0","type":"type"},{"doc":"A session.","ref":"Charon.Models.Session.html","title":"Charon.Models.Session","type":"module"},{"doc":"Upgrade a session (or map created from a session struct) to the latest struct version (7).","ref":"Charon.Models.Session.html#upgrade_version/2","title":"Charon.Models.Session.upgrade_version/2","type":"function"},{"doc":"@charon_config Charon.Config.from_enum(token_issuer: \"local\")\n\n    # old version without the :version, :refesh_expires_at fields but with :__struct__ set\n    # is updated to latest version (7)\n    iex> session = %{\n    ...>   __struct__: Session,\n    ...>   created_at: 0,\n    ...>   expires_at: 1,\n    ...>   extra_payload: %{},\n    ...>   id: \"ab\",\n    ...>   refresh_token_id: \"cd\",\n    ...>   refreshed_at: 15,\n    ...>   type: :full,\n    ...>   user_id: 9\n    ...> }\n    ...> |> upgrade_version(@charon_config)\n    iex> %Session{\n    ...>   created_at: 0,\n    ...>   expires_at: 1,\n    ...>   extra_payload: %{},\n    ...>   id: \"ab\",\n    ...>   prev_tokens_fresh_from: 0,\n    ...>   refresh_expires_at: 1,\n    ...>   refresh_token_id: \"cd\",\n    ...>   refreshed_at: 15,\n    ...>   tokens_fresh_from: 15,\n    ...>   type: :full,\n    ...>   user_id: 9,\n    ...>   version: 7\n    ...> } = session\n\n    # old version - with :expires_at = nil - is updated without error\n    iex> session = %{\n    ...>   __struct__: Session,\n    ...>   created_at: 0,\n    ...>   expires_at: nil,\n    ...>   extra_payload: %{},\n    ...>   id: \"ab\",\n    ...>   refresh_token_id: \"cd\",\n    ...>   refreshed_at: 15,\n    ...>   type: :full,\n    ...>   user_id: 9\n    ...> }\n    ...> |> upgrade_version(@charon_config)\n    iex> %Session{\n    ...>   created_at: 0,\n    ...>   expires_at: :infinite,\n    ...>   extra_payload: %{},\n    ...>   id: \"ab\",\n    ...>   prev_tokens_fresh_from: 0,\n    ...>   refresh_expires_at: refresh_exp,\n    ...>   refresh_token_id: \"cd\",\n    ...>   refreshed_at: 15,\n    ...>   tokens_fresh_from: 15,\n    ...>   type: :full,\n    ...>   user_id: 9,\n    ...>   version: 7\n    ...> } = session\n    iex> is_integer(refresh_exp) and refresh_exp >= System.os_time(:second)\n    true","ref":"Charon.Models.Session.html#upgrade_version/2-doctests","title":"DocTests - Charon.Models.Session.upgrade_version/2","type":"function"},{"doc":"","ref":"Charon.Models.Session.html#t:t/0","title":"Charon.Models.Session.t/0","type":"type"},{"doc":"Tokens to be communicated to the client.","ref":"Charon.Models.Tokens.html","title":"Charon.Models.Tokens","type":"module"},{"doc":"","ref":"Charon.Models.Tokens.html#t:t/0","title":"Charon.Models.Tokens.t/0","type":"type"},{"doc":"Plugs to create, update/refresh and delete sessions.\nWhen creating or updating a session, new tokens are created as well.","ref":"Charon.SessionPlugs.html","title":"Charon.SessionPlugs","type":"module"},{"doc":"Delete the persistent session identified by the session_id in the token claims.\n\nNote that the token remains valid until it expires, it is left up to the client to drop the access token. It will no longer be possible to refresh the session, however.","ref":"Charon.SessionPlugs.html#delete_session/2","title":"Charon.SessionPlugs.delete_session/2","type":"function"},{"doc":"# instructs browsers to clear signature cookies\n    iex> conn()\n    ...> |> Plug.Test.put_req_cookie(@config.access_cookie_name, \"anything\")\n    ...> |> Plug.Test.put_req_cookie(@config.refresh_cookie_name, \"anything\")\n    ...> |> delete_session(@config)\n    ...> |> Conn.fetch_cookies()\n    ...> |> Map.get(:cookies)\n    %{}","ref":"Charon.SessionPlugs.html#delete_session/2-examples-doctests","title":"Examples / doctests - Charon.SessionPlugs.delete_session/2","type":"function"},{"doc":"Create or update a session. If a session exists in the conn, the session is updated / refreshed, otherwise a new one is created.\nRefresh an existing session by putting it on the connection with `Charon.TokenPlugs.load_session/2`.\n\nIn both cases, new access / refresh tokens are created and stored in the conn's private map.\nThe server-side session stored in the session store is created / updated as well.\n\nIf a new session is created, options `:user_id` and `:token_transport` must be provided or an error will be raised.\n\nThe token transport can be:\n - `:cookie_only` - tokens are returned to the client as cookies\n - `:cookie` - tokens' signatures are split from the tokens and sent as http-only strictly-same-site secure cookies\n - `:bearer` - tokens are left as-is for the implementing application to return to the client\n\nIf config option `:enforce_browser_cookies` is enabled, browser clients will be attempted to be\ndetected by the presence of (forbidden) header \"Sec-Fetch-Mode\", in which case only cookie-based\ntoken transports will be allowed. This feature is experimental and disabled by default,\nalthough this may change in a future release.\n\nOptionally, it is possible to add extra claims to the access- and refresh tokens or to store extra payload in the server-side session.\n\nSession stores may return an optimistic locking error, meaning there are concurrent updates to a session.\nIn this case, `upsert/3` will raise a `Charon.SessionPlugs.SessionUpdateConflictError`, which should result\nin an HTTP 409 Conflict error.\nIf the session store returns another error, a `Charon.SessionPlugs.SessionStorageError` is raised,\nwhich is an unrecoverable state that should result in an HTTP 500 Internal Server Error.","ref":"Charon.SessionPlugs.html#upsert_session/3","title":"Charon.SessionPlugs.upsert_session/3","type":"function"},{"doc":"The following claims are set by default:\n- `\"exp\"` expires at (this value is guaranteed to never outlive the session itself)\n- `\"iat\"` time of token creation\n- `\"iss\"` issuer, usually a url like \"https://myapp.com\"\n- `\"jti\"` jwt id, random unique id for the token (a refresh token's id is stored in the session as well)\n- `\"nbf\"` not before, same value as `\"iat\"` but means \"token not valid before this time\"\n- `\"sid\"` session id\n- `\"sub\"` subject, the user id of the session owner\n- `\"type\"` type, `\"access\"` or `\"refresh\"`\n- `\"styp\"` session type\n\nAdditional claims or overrides can be provided with `opts`.","ref":"Charon.SessionPlugs.html#upsert_session/3-claims","title":"Claims - Charon.SessionPlugs.upsert_session/3","type":"function"},{"doc":"# error if user id not set for new session\n    iex> upsert_session(conn(), @config, token_transport: :bearer)\n    ** (RuntimeError) Set user id using upsert_session/3 option :user_id\n\n    # error if signature transport not set for new session\n    iex> upsert_session(conn(), @config, user_id: 1)\n    ** (RuntimeError) Set token transport using upsert_session/3 option :token_transport\n\n    # creates session if none present in conn\n    iex> conn = upsert_session(conn(), @config, user_id: 1, token_transport: :bearer)\n    iex> %Session{} = Utils.get_session(conn)\n    iex> %Tokens{} = Utils.get_tokens(conn)\n\n    # works with infinite lifespan sessions\n    iex> conn = upsert_session(conn(), %{@config | session_ttl: :infinite}, user_id: 1, token_transport: :bearer)\n    iex> %Session{expires_at: :infinite} = Utils.get_session(conn)\n    iex> %Tokens{} = Utils.get_tokens(conn)\n\n    # renews session if present in conn, updating only refresh_tokens, refreshed_at, and refresh_expires_at\n    # existing session's user id will not change despite attempted override\n    iex> old_session = test_session(user_id: 43, id: \"a\", expires_at: :infinite, refresh_expires_at: 0, refreshed_at: 0)\n    iex> conn = conn()\n    ...> |> Conn.put_private(@session, old_session)\n    ...> |> upsert_session(@config, user_id: 1, token_transport: :bearer)\n    iex> session = Utils.get_session(conn) |> Map.from_struct()\n    iex> old_session = Map.from_struct(old_session)\n    iex> Enum.find(~w(id user_id created_at expires_at)a, & session[&1] != old_session[&1])\n    nil\n    iex> Enum.find(~w(refresh_token_id refreshed_at refresh_expires_at)a, & session[&1] == old_session[&1])\n    nil\n\n    # returns token signatures in cookies if token transport is :cookie\n    # cookie options are merged with defaults\n    iex> conn = upsert_session(conn(), @config, user_id: 1, token_transport: :cookie)\n    iex> cookies = conn |> Conn.fetch_cookies() |> Map.get(:cookies)\n    iex> <<_access_sig::binary>> = Map.get(cookies, @config.access_cookie_name)\n    iex> <<_refresh_sig::binary>> = Map.get(cookies, @config.refresh_cookie_name)\n    iex> tokens = Utils.get_tokens(conn)\n    iex> [_, _] = String.split(tokens.access_token, \".\", trim: true)\n    iex> [_, _] = String.split(tokens.refresh_token, \".\", trim: true)\n    iex> %{http_only: true, path: _, same_site: \"Strict\", secure: true, max_age: _} = conn.resp_cookies[\"_refresh_token_signature\"]\n\n\n    # returns full tokens in cookies if token transport is :cookie_only\n    iex> conn = upsert_session(conn(), @config, user_id: 1, token_transport: :cookie_only)\n    iex> cookies = conn |> Conn.fetch_cookies() |> Map.get(:cookies)\n    iex> [_, _, _] = cookies |> Map.get(@config.access_cookie_name) |> String.split(\".\", trim: true)\n    iex> [_, _, _] = cookies |> Map.get(@config.refresh_cookie_name) |> String.split(\".\", trim: true)\n    iex> %{access_token: nil, refresh_token: nil} = Utils.get_tokens(conn)\n\n    # tokens get a lot of default claims\n    iex> conn = upsert_session(conn(), @config, user_id: 1, token_transport: :bearer)\n    iex> %{\"exp\" => _, \"iat\" => _, \"iss\" => \"my_test_app\", \"jti\" => <<_::binary>>, \"nbf\" => _, \"sid\" => < >, \"sub\" => 1, \"type\" => \"access\", \"styp\" => \"full\"} = get_private(conn, @access_token_payload)\n    iex> %{\"exp\" => _, \"iat\" => _, \"iss\" => \"my_test_app\", \"jti\" => <<_::binary>>, \"nbf\" => _, \"sid\" => ^sid, \"sub\" => 1, \"type\" => \"refresh\", \"styp\" => \"full\"} = get_private(conn, @refresh_token_payload)\n\n    # allows adding extra claims to tokens\n    iex> conn = upsert_session(\n    ...>   conn(),\n    ...>   @config,\n    ...>   user_id: 1,\n    ...>   token_transport: :bearer,\n    ...>   access_claim_overrides: %{\"much\" => :extra},\n    ...>   refresh_claim_overrides: %{\"really\" => true}\n    ...> )\n    iex> %{\"much\" => :extra} = get_private(conn, @access_token_payload)\n    iex> %{\"really\" => true} = get_private(conn, @refresh_token_payload)\n\n    # allows adding extra payload to session\n    iex> conn = upsert_session(\n    ...>   conn(),\n    ...>   @config,\n    ...>   user_id: 1,\n    ...>   token_transport: :bearer,\n    ...>   extra_session_payload: %{what?: \"that's right!\"}\n    ...> )\n    iex> %Session{extra_payload: %{what?: \"that's right!\"}} = Utils.get_session(conn)\n\n    # allows separating sessions by type (default :full)\n    iex> conn = upsert_session(conn(), @config, session_type: :oauth2, user_id: 1, token_transport: :bearer)\n    iex> %Session{type: :oauth2} = Utils.get_session(conn)\n    iex> %{\"styp\" => \"oauth2\"} = get_private(conn, @access_token_payload)","ref":"Charon.SessionPlugs.html#upsert_session/3-examples-doctests","title":"Examples / doctests - Charon.SessionPlugs.upsert_session/3","type":"function"},{"doc":"","ref":"Charon.SessionPlugs.html#t:upsert_session_opts/0","title":"Charon.SessionPlugs.upsert_session_opts/0","type":"type"},{"doc":"The persistent session store. See `Charon.SessionStore.Behaviour` for callbacks.\nAll functions delegate to the configured session store module.\n\nClient applications should use this module to interact with the underlying implementation,\nbecause this module takes care of session struct version upgrades as well.","ref":"Charon.SessionStore.html","title":"Charon.SessionStore","type":"module"},{"doc":"Behaviour definition of a persistent session store.\nClients should not use the implementation directly, but should use `Charon.SessionStore`.","ref":"Charon.SessionStore.Behaviour.html","title":"Charon.SessionStore.Behaviour","type":"behaviour"},{"doc":"Implementations are expected to store sessions by ID, user ID and session type.\nFor the optional callbacks `get_all/3` and `delete_all/3`, sessions should be retrievable\nby user ID and session type only.\n\nImplementations should return the session exactly as it went in\n(they don't have to take care of struct version upgrades).\n\nImplementations should handle cleanup of expired entries,\nbut may define additional functions and instructions to take care of such things\n(like a `cleanup/0` that should run periodically).\n\nImplementations should implement optimistic locking with respect to the `:lock_version` field\nof the session struct. On an update, the lock version of the updated session should match\nthe lock version of the existing session. If that is the case, the implementation should increase\n`:lock_version` and proceed with the update. If not, the implementation should abort the update\nand return `{:error, :conflict}`.","ref":"Charon.SessionStore.Behaviour.html#module-implementation-guidelines","title":"Implementation guidelines - Charon.SessionStore.Behaviour","type":"behaviour"},{"doc":"Delete session of type `type` with id `session_id` for user with id `user_id`.\n\nImplementations may choose to ignore `user_id`, since `session_id` is unique by itself.","ref":"Charon.SessionStore.Behaviour.html#c:delete/4","title":"Charon.SessionStore.Behaviour.delete/4","type":"callback"},{"doc":"Delete all sessions of type `type` for the user with id `user_id`.","ref":"Charon.SessionStore.Behaviour.html#c:delete_all/3","title":"Charon.SessionStore.Behaviour.delete_all/3","type":"callback"},{"doc":"Get session of type `type` with id `session_id` for user with id `user_id`.\nMust not return sessions that have expired,\nor that can't be refreshed anymore because the refresh token has expired.\n\nImplementations may choose to ignore `user_id`, since `session_id` is unique by itself.","ref":"Charon.SessionStore.Behaviour.html#c:get/4","title":"Charon.SessionStore.Behaviour.get/4","type":"callback"},{"doc":"Get all sessions of type `type` for the user with id `user_id`.\nMust not return sessions that have expired,\nor that can't be refreshed anymore because the refresh token has expired.","ref":"Charon.SessionStore.Behaviour.html#c:get_all/3","title":"Charon.SessionStore.Behaviour.get_all/3","type":"callback"},{"doc":"Insert or update `session`. If the implementation supports optimistic locking, it\nwill return `{:error, :conflict}` on an optimistic locking conflict.\n\nValues `session_id`, `user_id` and `type` are taken from the `session` struct.\nImplementations may choose to ignore `user_id`, since `session_id` is unique by itself.\n\nImplementations may assume that `Charon.SessionPlugs.upsert_session/3` will ensure that:\n - `:refreshed_at` contains the current unix timestamp\n - `:refresh_expires_at` never exceeds `:expires_at`","ref":"Charon.SessionStore.Behaviour.html#c:upsert/2","title":"Charon.SessionStore.Behaviour.upsert/2","type":"callback"},{"doc":"A dummy session store that can be used if fully stateless tokens are desired.","ref":"Charon.SessionStore.DummyStore.html","title":"Charon.SessionStore.DummyStore","type":"module"},{"doc":"An in-memory persistent session store, implements behaviour `Charon.SessionStore`.\nIn addition to the required callbacks, this store also provides `get_all/3` and `delete_all/3` (for a user) functions.","ref":"Charon.SessionStore.LocalStore.html","title":"Charon.SessionStore.LocalStore","type":"module"},{"doc":"Add this store to your supervision tree to use it.\n\n    children = [\n      Charon.SessionStore.LocalStore\n    ]","ref":"Charon.SessionStore.LocalStore.html#module-usage","title":"Usage - Charon.SessionStore.LocalStore","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Charon.SessionStore.LocalStore.html#child_spec/1","title":"Charon.SessionStore.LocalStore.child_spec/1","type":"function"},{"doc":"","ref":"Charon.SessionStore.LocalStore.html#start_link/1","title":"Charon.SessionStore.LocalStore.start_link/1","type":"function"},{"doc":"A persistent session store based on Redis, which implements behaviour `Charon.SessionStore`.\nIn addition to the required callbacks, this store also provides `get_all/3` and `delete_all/3` (for a user) functions.","ref":"Charon.SessionStore.RedisStore.html","title":"Charon.SessionStore.RedisStore","type":"module"},{"doc":"This module needs a Redis >= 7.0.0 instance and needs permissions to create Redis functions.\nThe optimistic-locking functionality of the store was not designed with a Redis cluster in mind\nand will behave unpredictably when used with a distributed Redis deployment.\nUsing a failover-replica should be fine, however.\nThe Redis functions are registered with a name that includes the hashed Charon version,\nto make sure that the called function matches the expected code,\nand multiple Charon deployments can share a Redis instance.","ref":"Charon.SessionStore.RedisStore.html#module-redis-requirements","title":"Redis requirements - Charon.SessionStore.RedisStore","type":"module"},{"doc":"Additional config is required for this module (see `Charon.SessionStore.RedisStore.Config`):\n\n    Charon.Config.from_enum(\n      ...,\n      optional_modules: %{\n        Charon.SessionStore.RedisStore => %{\n          key_prefix: \"charon_\",\n          get_signing_key: &RedisStore.default_signing_key/1\n        }\n      }\n    )\n\nThe following options are supported:\n  - `:key_prefix` (optional). A string prefix for the Redis keys that are sessions.\n  - `:get_signing_key` (optional). A getter/1 that returns the key that is used to sign and verify serialized session binaries.","ref":"Charon.SessionStore.RedisStore.html#module-config","title":"Config - Charon.SessionStore.RedisStore","type":"module"},{"doc":"RedisStore uses a connection pool and has to register Redis functions on startup.\nIn order to initialize it, you must add RedisStore to your application's supervision tree.\n\n    # in application.ex\n    def start(_, ) do\n      redix_opts = [host: \"localhost\", port: 6379, password: \"supersecret\", database: 0]\n\n      children = [\n        ...\n        {Charon.SessionStore.RedisStore, pool_size: 15, redix_opts: redix_opts},\n        ...\n      ]\n\n      opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n      Supervisor.start_link(children, opts)\n    end","ref":"Charon.SessionStore.RedisStore.html#module-initialize-store","title":"Initialize store - Charon.SessionStore.RedisStore","type":"module"},{"doc":"- `:name` (default module name) name of the supervisor\n  - `:pool_size` (default 10) total number of (local) normal workers\n  - `:pool_max_overflow` (default 5) max number of (local) extra workers in case of high load\n  - `:redix_opts` passed to `Redix.start_link/1`\n\nThe pool and its config options are local to the Elixir/OTP node,\nso if you use multiple nodes, the total connection count to Redis maxes out at\n`(pool_size + pool_max_overflow) * number_of_nodes`.","ref":"Charon.SessionStore.RedisStore.html#module-options","title":"Options - Charon.SessionStore.RedisStore","type":"module"},{"doc":"In order to offer some defense-in-depth against a compromised Redis server,\nthe serialized sessions are signed using a HMAC. The key is derived from the Charon\nbase secret, but can be overridden in the config. This is not usually necessary, except\nwhen you're changing the base secret and still want to access your sessions.\nIt is debatable, of course, in case your Redis server is compromised,\nif your application server holding the signing key can still be considered secure.\nHowever, given that there are no real drawbacks to using signed session binaries,\nsince the performance cost is negligible, no extra config is needed, it is easy to implement,\nand defense-in-depth is a good guiding principle, RedisStore implements this feature anyway.","ref":"Charon.SessionStore.RedisStore.html#module-session-signing","title":"Session signing - Charon.SessionStore.RedisStore","type":"module"},{"doc":"RedisStore stores sessions until their associated refresh token(s) expire.\nThat means that `:refresh_expires_at` is used to determine if a session is \"alive\",\nnot `:expires_at`.\nThis makes sense, because a session without a valid refresh token is effectively useless,\nand it means we can support \"infinite lifetime\" sessions while still pruning sessions that aren't used.\nThis is one reason to set `:refresh_ttl` to a limited value, no more than six months is recommended.\n\nLast but not least, [Redis 7 functions](https://redis.io/docs/manual/programmability/functions-intro/)\nare used to implement some features, specifically optimistic locking, and to ensure all callbacks\nuse only a single round-trip to the Redis instance.","ref":"Charon.SessionStore.RedisStore.html#module-implementation-details","title":"Implementation details - Charon.SessionStore.RedisStore","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Charon.SessionStore.RedisStore.html#child_spec/1","title":"Charon.SessionStore.RedisStore.child_spec/1","type":"function"},{"doc":"Get the default session signing key that is used if config option `:get_signing_key` is not set explicitly.","ref":"Charon.SessionStore.RedisStore.html#default_signing_key/1","title":"Charon.SessionStore.RedisStore.default_signing_key/1","type":"function"},{"doc":"Start the RedisStore supervisor, which registeres all required Redis functions and initiates the connection pool.","ref":"Charon.SessionStore.RedisStore.html#start_link/1","title":"Charon.SessionStore.RedisStore.start_link/1","type":"function"},{"doc":"- `:name` (default module name) name of the supervisor\n  - `:pool_size` (default 10) total number of (local) normal workers\n  - `:pool_max_overflow` (default 5) max number of (local) extra workers in case of high load\n  - `:redix_opts` passed to `Redix.start_link/1`\n\nThe pool and its config options are local to the Elixir/OTP node,\nso if you use multiple nodes, the total connection count to Redis maxes out at\n`(pool_size + pool_max_overflow) * number_of_nodes`.","ref":"Charon.SessionStore.RedisStore.html#start_link/1-options","title":"Options - Charon.SessionStore.RedisStore.start_link/1","type":"function"},{"doc":"Config module for `Charon.SessionStore.RedisStore`.","ref":"Charon.SessionStore.RedisStore.Config.html","title":"Charon.SessionStore.RedisStore.Config","type":"module"},{"doc":"Build config struct from enumerable (useful for passing in application environment).\nRaises for missing mandatory keys and sets defaults for optional keys.","ref":"Charon.SessionStore.RedisStore.Config.html#from_enum/1","title":"Charon.SessionStore.RedisStore.Config.from_enum/1","type":"function"},{"doc":"Get the config for this module from the parent `Charon.Config` struct.","ref":"Charon.SessionStore.RedisStore.Config.html#get_mod_config/1","title":"Charon.SessionStore.RedisStore.Config.get_mod_config/1","type":"function"},{"doc":"","ref":"Charon.SessionStore.RedisStore.Config.html#t:t/0","title":"Charon.SessionStore.RedisStore.Config.t/0","type":"type"},{"doc":"Utility functions for writing tests.","ref":"Charon.TestHelpers.html","title":"Charon.TestHelpers","type":"module"},{"doc":"Create a test session and return the session, a set of tokens for it and the response cookies.","ref":"Charon.TestHelpers.html#create_session/2","title":"Charon.TestHelpers.create_session/2","type":"function"},{"doc":"- `:upsert_session_opts` (default `[]`) as defined in `t:Charon.SessionPlugs.upsert_session_opts/0`","ref":"Charon.TestHelpers.html#create_session/2-options","title":"Options - Charon.TestHelpers.create_session/2","type":"function"},{"doc":"Override configuration for an optional module.","ref":"Charon.TestHelpers.html#override_opt_mod_conf/3","title":"Charon.TestHelpers.override_opt_mod_conf/3","type":"function"},{"doc":"Create a new test session and put a token (and its cookie when needed) on the conn.\nThis is essentially `create_session/3` and `put_token_for/3` combined into one\nconvenience function.","ref":"Charon.TestHelpers.html#put_token/3","title":"Charon.TestHelpers.put_token/3","type":"function"},{"doc":"The upsert_session_opts defined in `t:Charon.SessionPlugs.upsert_session_opts/0` AND\n  - `:token` is either `:access` (default) or `:refresh`","ref":"Charon.TestHelpers.html#put_token/3-options","title":"Options - Charon.TestHelpers.put_token/3","type":"function"},{"doc":"Put a token (and its cookie when needed) from the test session on the conn.\nUse `create_session/3` to create the test session.","ref":"Charon.TestHelpers.html#put_token_for/3","title":"Charon.TestHelpers.put_token_for/3","type":"function"},{"doc":"- `:token` is either `:access` (default) or `:refresh`","ref":"Charon.TestHelpers.html#put_token_for/3-options","title":"Options - Charon.TestHelpers.put_token_for/3","type":"function"},{"doc":"","ref":"Charon.TestHelpers.html#t:test_session/0","title":"Charon.TestHelpers.test_session/0","type":"type"},{"doc":"Entrypoint for `Charon.TokenFactory.Behaviour` implementation.\nAll functions delegate to the configured module.","ref":"Charon.TokenFactory.html","title":"Charon.TokenFactory","type":"module"},{"doc":"Behaviour for token-signing modules.\n\nNote that the token payload must be returned as a map with string keys on verification.\nWhen the payload is serialized as JSON, this happens automatically.\nHowever, when Erlang term format is used, this is not the case.","ref":"Charon.TokenFactory.Behaviour.html","title":"Charon.TokenFactory.Behaviour","type":"behaviour"},{"doc":"Create a new token with the provided payload and a valid signature.","ref":"Charon.TokenFactory.Behaviour.html#c:sign/2","title":"Charon.TokenFactory.Behaviour.sign/2","type":"callback"},{"doc":"Verify that the signature matches the token's header and payload, and decode the payload.\n\nMust return a map of string keys.","ref":"Charon.TokenFactory.Behaviour.html#c:verify/2","title":"Charon.TokenFactory.Behaviour.verify/2","type":"callback"},{"doc":"JWT's with either symmetric (HMAC) or asymmetric (EDDSA) signatures.\nThe default, simplest and most performant option is symmetric signatures (MAC),\nwith the key derived from the Charon base secret.\n\nAsymmetric tokens can be used when it is desirable for an external party\nto be able to verify a token's integrity,\nin which case distributing symmetric keys can be a hassle and a security risk.","ref":"Charon.TokenFactory.Jwt.html","title":"Charon.TokenFactory.Jwt","type":"module"},{"doc":"In order to sign and verify JWT's, a keyset is used.\nA keyset is a map of key ID's to keys.\nA key is a tuple of the signing algorithm and the actual secret(s).\nTo simplify things and discourage key reuse,\na key can only be used with a single signing algorithm.\nThe default keyset looks like this, for example:\n\n    %{\"default\" => {:hmac_sha256, <<0, ...>>}}\n\nEvery token that is signed gets a `\"kid\"` claim in its header, allowing it to\nbe verified with the specific key and algorithm that it was signed with.","ref":"Charon.TokenFactory.Jwt.html#module-keysets","title":"Keysets - Charon.TokenFactory.Jwt","type":"module"},{"doc":"It is possible to transition to a new signing key by adding a new key to the keyset\nand setting it as the new signing key using the `:signing_key` config option:\n\n    %{\n      \"default\" => {:hmac_sha256, <<0, ...>>},\n      \"new!\" => {:hmac_sha512, <<1, ...>>}\n    }\n\nOlder tokens will be verified using the older key, based on their `\"kid\"` header claim.","ref":"Charon.TokenFactory.Jwt.html#module-key-cycling","title":"Key cycling - Charon.TokenFactory.Jwt","type":"module"},{"doc":"Legacy or external tokens may not have a `\"kid\"` header claim.\nSuch tokens can still be verified by adding\na `\"kid_not_set. \"` (for example \"kid_not_set.HS256\")\nkey to the keyset.","ref":"Charon.TokenFactory.Jwt.html#module-tokens-without-a-kid-header-claim","title":"Tokens without a `\"kid\"` header claim - Charon.TokenFactory.Jwt","type":"module"},{"doc":"Symmetric signatures are message authentication codes or MACs,\neither HMACs based on SHA256, 384 or 512,\nor a MAC generated using Blake3's [keyed-hashing mode](https://docs.rs/blake3/latest/blake3/fn.keyed_hash.html),\nwhich can be used directly without using a HMAC wrapper.\nUsing Blake3 requires the optional dependency [Blake3](https://hex.pm/packages/blake3)\nand a key of exactly 256 bits.\n\nBy default, a SHA256-based HMAC is used.","ref":"Charon.TokenFactory.Jwt.html#module-symmetric-signatures","title":"Symmetric signatures - Charon.TokenFactory.Jwt","type":"module"},{"doc":"Asymmetric signatures are created using EDDSA (Edwards-curve Digital Signature Algorithm)\nbased on Curve25519 or Curve448.\nUse in JWTs is standardized (pending) in [RFC 8073](https://datatracker.ietf.org/doc/rfc8037/).\nThese algorithms were chosen for performance and implementation ease,\nsince they offer built-in protection against many side-channel (timing) attacks and are\nnot susceptible to nonce-reuse (technically, they are, but not on the part of the implementation,\nwhich means they are safe to use in your [PlayStation](https://en.wikipedia.org/wiki/EdDSA#Secure_coding)).\nUnless you are paranoid, use Curve25519, which offers about 128 bits of security.\nCurve448 offers about 224 bits, but is significantly slower.\n\nIn order to use asymmetric signatures, generate a key using `gen_keypair/1`.\nCreate a publishable JWK using `keypair_to_pub_jwk/1`.\n\n    iex> keypair = Jwt.gen_keypair(:eddsa_ed25519)\n    iex> {:eddsa_ed25519, {_pubkey, _privkey}} = keypair\n    iex> %{\"crv\" => \"Ed25519\", \"kty\" => \"OKP\", \"x\" => <<_::binary>>} = Jwt.keypair_to_pub_jwk(keypair)","ref":"Charon.TokenFactory.Jwt.html#module-asymmetric-signatures","title":"Asymmetric signatures - Charon.TokenFactory.Jwt","type":"module"},{"doc":"Additional config is required for this module (see `Charon.TokenFactory.Jwt.Config`):\n\n    Charon.Config.from_enum(\n      ...,\n      optional_modules: %{\n        Charon.TokenFactory.Jwt => %{\n          get_keyset: fn -> %{\"key1\" => {:hmac_sha256, \"my_key\"}} end,\n          signing_key: \"key1\"\n        }\n      }\n    )\n\nThe following options are supported:\n  - `:get_keyset` (optional, default `default_keyset/1`). The keyset used to sign and verify JWTs. If not specified, a default keyset with a single key called \"default\" is used, which is derived from Charon's base secret.\n  - `:signing_key` (optional, default \"default\"). The ID of the key in the keyset that is used to sign new tokens.","ref":"Charon.TokenFactory.Jwt.html#module-config","title":"Config - Charon.TokenFactory.Jwt","type":"module"},{"doc":"# gracefully handles malformed tokens / unsupported algo's / invalid signature\n    iex> verify(\"a\", @charon_config)\n    {:error, \"malformed token\"}\n    iex> verify(\"a.b.c\", @charon_config)\n    {:error, \"encoding invalid\"}\n    iex> header = \"notjson\" |> url_encode()\n    iex> verify(header <> \".YQ.YQ\", @charon_config)\n    {:error, \"json invalid\"}\n    iex> header = %{\"missing\" => \"alg\"} |> Jason.encode!() |> url_encode()\n    iex> verify(header <> \".YQ.YQ\", @charon_config)\n    {:error, \"malformed header\"}\n    iex> header = %{\"alg\" => \"boom\"} |> Jason.encode!() |> url_encode()\n    iex> verify(header <> \".YQ.YQ\", @charon_config)\n    {:error, \"key not found\"}\n    iex> header = %{\"alg\" => \"HS256\", \"kid\" => \"default\"} |> Jason.encode!() |> url_encode()\n    iex> verify(header <> \".YQ.YQ\", @charon_config)\n    {:error, \"signature invalid\"}\n\n    # supports cycling to a new signing key, while still verifying old tokens\n    iex> {:ok, token} = sign(%{}, @charon_config)\n    iex> keyset = Jwt.default_keyset(@charon_config)\n    iex> keyset = Map.put(keyset, \"ed25519_1\", Jwt.gen_keypair(:eddsa_ed25519))\n    iex> config = override_opt_mod_conf(@charon_config, Jwt, get_keyset: fn _ -> keyset end, signing_key: \"ed25519_1\")\n    iex> {:ok, _} = verify(token, config)\n    iex> {:ok, new_token} = sign(%{}, config)\n    iex> new_token == token\n    false\n\n    # an old / external / legacy token without a \"kid\" claim can still be verified\n    # by adding a \"kid_not_set. \" key to the keyset\n    # a token MUST have an alg claim, which is mandatory according to the JWT spec\n    iex> [header, pl] = [%{\"alg\" => \"HS256\"}, %{}] |> Enum.map(&Jason.encode!/1) |> Enum.map(&url_encode/1)\n    iex> base = \"#{header}.#{pl}\"\n    iex> key = :crypto.strong_rand_bytes(32)\n    iex> signature = :crypto.mac(:hmac, :sha256, key, base) |> url_encode()\n    iex> token = \"#{base}.#{signature}\"\n    iex> {:error, \"key not found\"} = verify(token, @charon_config)\n    iex> keyset = %{\"kid_not_set.HS256\" => {:hmac_sha256, key}}\n    iex> config = override_opt_mod_conf(@charon_config, Jwt, get_keyset: fn _ -> keyset end)\n    iex> {:ok, _} = verify(token, config)","ref":"Charon.TokenFactory.Jwt.html#module-examples-doctests","title":"Examples / doctests - Charon.TokenFactory.Jwt","type":"module"},{"doc":"Get the default keyset that is used if config option `:get_keyset` is not set explicitly.","ref":"Charon.TokenFactory.Jwt.html#default_keyset/1","title":"Charon.TokenFactory.Jwt.default_keyset/1","type":"function"},{"doc":"Generate a new keypair for an asymmetrically signed JWT.","ref":"Charon.TokenFactory.Jwt.html#gen_keypair/1","title":"Charon.TokenFactory.Jwt.gen_keypair/1","type":"function"},{"doc":"Convert a keypair generated by `gen_keypair/1` to a publishable JWK\ncontaining only the public key.","ref":"Charon.TokenFactory.Jwt.html#keypair_to_pub_jwk/1","title":"Charon.TokenFactory.Jwt.keypair_to_pub_jwk/1","type":"function"},{"doc":"","ref":"Charon.TokenFactory.Jwt.html#t:eddsa_alg/0","title":"Charon.TokenFactory.Jwt.eddsa_alg/0","type":"type"},{"doc":"","ref":"Charon.TokenFactory.Jwt.html#t:eddsa_keypair/0","title":"Charon.TokenFactory.Jwt.eddsa_keypair/0","type":"type"},{"doc":"","ref":"Charon.TokenFactory.Jwt.html#t:hmac_alg/0","title":"Charon.TokenFactory.Jwt.hmac_alg/0","type":"type"},{"doc":"","ref":"Charon.TokenFactory.Jwt.html#t:key/0","title":"Charon.TokenFactory.Jwt.key/0","type":"type"},{"doc":"","ref":"Charon.TokenFactory.Jwt.html#t:keyset/0","title":"Charon.TokenFactory.Jwt.keyset/0","type":"type"},{"doc":"","ref":"Charon.TokenFactory.Jwt.html#t:mac_alg/0","title":"Charon.TokenFactory.Jwt.mac_alg/0","type":"type"},{"doc":"","ref":"Charon.TokenFactory.Jwt.html#t:symmetric_key/0","title":"Charon.TokenFactory.Jwt.symmetric_key/0","type":"type"},{"doc":"Config module for `Charon.TokenFactory.Jwt`.","ref":"Charon.TokenFactory.Jwt.Config.html","title":"Charon.TokenFactory.Jwt.Config","type":"module"},{"doc":"Build config struct from enumerable (useful for passing in application environment).\nRaises for missing mandatory keys and sets defaults for optional keys.","ref":"Charon.TokenFactory.Jwt.Config.html#from_enum/1","title":"Charon.TokenFactory.Jwt.Config.from_enum/1","type":"function"},{"doc":"Get the config for this module from the parent `Charon.Config` struct.","ref":"Charon.TokenFactory.Jwt.Config.html#get_mod_config/1","title":"Charon.TokenFactory.Jwt.Config.get_mod_config/1","type":"function"},{"doc":"","ref":"Charon.TokenFactory.Jwt.Config.html#t:t/0","title":"Charon.TokenFactory.Jwt.Config.t/0","type":"type"},{"doc":"The plugs in this module (and its submodules) can be used to verify tokens.\nThe token's presence, signature, expiration and any claims can be checked.\nAdditionally, the token's session can be loaded and, in case of a refresh token,\nit can be verified that it matches the session.\n\nIn case of validation errors, the plugs add an \"auth error\" to the conn,\nbut don't raise or halt the connection immediately.\nThis property can be used to support endpoints\nthat work with- or without authentication, for example,\nor if you want to support multiple kinds of tokens.\nThe plug `verify_no_auth_error/2` can be used to actually do something if there is an error.\nAll the plugs short-circuit, meaning that they immediately\nreturn the connection if there are errors.\n\nUsing the plugs in these module, you can construct your own verification pipeline\nusing either Plug.Builder or standard Phoenix router pipelines.\nHere are two examples for access- and refresh tokens, respectively,\nthat should be a good baseline for your own pipelines.","ref":"Charon.TokenPlugs.html","title":"Charon.TokenPlugs","type":"module"},{"doc":"defmodule MyApp.AccessTokenPipeline do\n      use Plug.Builder\n\n      @config Charon.Config.from_enum(Application.compile_env!(:my_app, :charon))\n\n      plug :get_token_from_auth_header\n      plug :get_token_from_cookie, @config.access_cookie_name\n      plug :verify_token_signature, @config\n      plug :verify_token_nbf_claim\n      plug :verify_token_exp_claim\n      plug :verify_token_claim_equals, {\"type\", \"access\"}\n      plug :verify_no_auth_error, &MyApp.TokenErrorHandler.on_error/2\n      plug Charon.TokenPlugs.PutAssigns\n    end","ref":"Charon.TokenPlugs.html#module-access-tokens","title":"Access tokens - Charon.TokenPlugs","type":"module"},{"doc":"defmodule MyApp.RefreshTokenPipeline do\n      use Plug.Builder\n\n      @config Charon.Config.from_enum(Application.compile_env!(:my_app, :charon))\n\n      plug :get_token_from_auth_header\n      plug :get_token_from_cookie, @config.refresh_cookie_name\n      plug :verify_token_signature, @config\n      plug :verify_token_nbf_claim\n      plug :verify_token_exp_claim\n      plug :verify_token_claim_equals, {\"type\", \"refresh\"}\n      plug :load_session, @config\n      plug :verify_token_fresh, 10\n      plug :verify_no_auth_error, &MyApp.TokenErrorHandler.on_error/2\n      plug Charon.TokenPlugs.PutAssigns\n    end","ref":"Charon.TokenPlugs.html#module-refresh-tokens","title":"Refresh tokens - Charon.TokenPlugs","type":"module"},{"doc":"Get a bearer token from the `authorization` header.","ref":"Charon.TokenPlugs.html#get_token_from_auth_header/2","title":"Charon.TokenPlugs.get_token_from_auth_header/2","type":"function"},{"doc":"iex> conn = conn() |> put_req_header(\"authorization\", \"Bearer aaa\")\n    iex> conn |> get_token_from_auth_header([]) |> Utils.get_auth_error()\n    nil\n    iex> conn |> get_token_from_auth_header([]) |> Utils.get_token_transport()\n    :bearer\n    iex> conn |> get_token_from_auth_header([]) |> Utils.get_bearer_token()\n    \"aaa\"\n\n    # missing auth header\n    iex> conn = conn()\n    iex> conn |> get_token_from_auth_header([]) |> Utils.get_auth_error()\n    nil\n\n    # auth header format must be correct\n    iex> conn = conn() |> put_req_header(\"authorization\", \"boom\")\n    iex> conn |> get_token_from_auth_header([]) |> Utils.get_bearer_token()\n    nil\n    iex> conn = conn() |> put_req_header(\"authorization\", \"Bearer \")\n    iex> conn |> get_token_from_auth_header([]) |> Utils.get_auth_error()\n    nil","ref":"Charon.TokenPlugs.html#get_token_from_auth_header/2-doctests","title":"Doctests - Charon.TokenPlugs.get_token_from_auth_header/2","type":"function"},{"doc":"Get the token or token signature from a cookie, if:\n - no bearer token was previously found by `get_token_from_auth_header/2`\n - OR the bearer token ends with \".\", in which case the cookie contents are appended to it","ref":"Charon.TokenPlugs.html#get_token_from_cookie/2","title":"Charon.TokenPlugs.get_token_from_cookie/2","type":"function"},{"doc":"# cookie is appended to a bearer token that ends with \".\"\n    iex> conn = conn() |> set_token(\"token.\") |> put_req_cookie(\"c\", \"sig\") |> fetch_cookies()\n    iex> conn = conn |> get_token_from_cookie(\"c\")\n    iex> conn |> Utils.get_token_transport()\n    :cookie\n    iex> conn |> Utils.get_bearer_token()\n    \"token.sig\"\n\n    # cookie is ignored if a bearer token is present that does not end with \".\"\n    iex> conn = conn() |> set_token(\"token\") |> put_req_cookie(\"c\", \"sig\") |> fetch_cookies()\n    iex> conn = conn |> get_token_from_cookie(\"c\")\n    iex> conn |> Utils.get_token_transport()\n    nil\n    iex> conn |> Utils.get_bearer_token()\n    \"token\"\n\n    # cookie contents are used as token if no bearer token was found previously\n    iex> conn = conn() |> put_req_cookie(\"c\", \"cookie token\") |> fetch_cookies()\n    iex> conn = conn |> get_token_from_cookie(\"c\")\n    iex> conn |> Utils.get_token_transport()\n    :cookie_only\n    iex> conn |> Utils.get_bearer_token()\n    \"cookie token\"","ref":"Charon.TokenPlugs.html#get_token_from_cookie/2-doctests","title":"Doctests - Charon.TokenPlugs.get_token_from_cookie/2","type":"function"},{"doc":"Appends the specified cookie's content to the bearer token, if the cookie is present and the token ends with a \".\" character.\nMust be used after `get_token_from_auth_header/2`.","ref":"Charon.TokenPlugs.html#get_token_sig_from_cookie/2","title":"Charon.TokenPlugs.get_token_sig_from_cookie/2","type":"function"},{"doc":"iex> conn = conn() |> set_token(\"token.\") |> put_req_cookie(\"c\", \"sig\") |> fetch_cookies()\n    iex> conn = conn |> get_token_from_cookie(\"c\")\n    iex> conn |> Utils.get_token_transport()\n    :cookie\n    iex> conn |> Utils.get_bearer_token()\n    \"token.sig\"\n\n    # cookie is ignored if bearer token does not end with .\n    iex> conn = conn() |> set_token(\"token\") |> put_req_cookie(\"c\", \"sig\") |> fetch_cookies()\n    iex> conn = conn |> get_token_from_cookie(\"c\")\n    iex> conn |> Utils.get_token_transport()\n    nil\n    iex> conn |> Utils.get_bearer_token()\n    \"token\"","ref":"Charon.TokenPlugs.html#get_token_sig_from_cookie/2-doctests","title":"Doctests - Charon.TokenPlugs.get_token_sig_from_cookie/2","type":"function"},{"doc":"Fetch the session to which the bearer token belongs.\nRaises on session store error.\nMust be used after `verify_token_signature/2`.","ref":"Charon.TokenPlugs.html#load_session/2","title":"Charon.TokenPlugs.load_session/2","type":"function"},{"doc":"iex> SessionStore.upsert(test_session(refresh_expires_at: 999999999999999), @config)\n    iex> conn = conn() |> set_token_payload(%{\"sid\" => \"a\", \"sub\" => 1, \"styp\" => \"full\"})\n    iex> %Session{} = conn |> load_session(@config) |> Internal.get_private(@session)\n\n    # token payload must contain \"sub\", \"sid\" and \"styp\" claims\n    iex> conn = conn() |> set_token_payload(1)\n    iex> conn |> load_session(@config) |> Utils.get_auth_error()\n    \"bearer token claim sub, sid or styp not found\"\n\n    # session must be found\n    iex> conn = conn() |> set_token_payload(%{\"sid\" => \"a\", \"sub\" => 1, \"styp\" => \"full\"})\n    iex> conn |> load_session(@config) |> Utils.get_auth_error()\n    \"session not found\"\n\n    iex> conn() |> load_session(@config)\n    ** (RuntimeError) must be used after verify_token_signature/2","ref":"Charon.TokenPlugs.html#load_session/2-doctests","title":"Doctests - Charon.TokenPlugs.load_session/2","type":"function"},{"doc":"Make sure that no previous plug of this module added an auth error.\nIn case of an error, `on_error` is called (it should probably halt the connection).","ref":"Charon.TokenPlugs.html#verify_no_auth_error/2","title":"Charon.TokenPlugs.verify_no_auth_error/2","type":"function"},{"doc":"iex> conn = conn()\n    iex> ^conn = verify_no_auth_error(conn, fn _conn, _error -> \"BOOM\" end)\n\n    # on error, send an error response\n    iex> conn = conn() |> set_auth_error(\"oops!\")\n    iex> conn = verify_no_auth_error(conn, & &1 |> send_resp(401, &2) |> halt())\n    iex> conn.halted\n    true\n    iex> conn.resp_body\n    \"oops!\"","ref":"Charon.TokenPlugs.html#verify_no_auth_error/2-doctests","title":"Doctests - Charon.TokenPlugs.verify_no_auth_error/2","type":"function"},{"doc":"Generically verify the bearer token payload.\nThe validation function `func` must return the conn or an error message.\nMust be used after `load_session/2`.","ref":"Charon.TokenPlugs.html#verify_session_payload/2","title":"Charon.TokenPlugs.verify_session_payload/2","type":"function"},{"doc":"iex> conn = conn() |> set_session(%{the: \"session\"})\n    iex> ^conn = conn |> verify_session_payload(fn conn, %{the: \"session\"} -> conn end)\n\n    # invalid\n    iex> conn = conn() |> set_session(%{the: \"session\"})\n    iex> conn |> verify_session_payload(fn _conn, s -> s[:missing] || \"invalid\" end) |> Utils.get_auth_error()\n    \"invalid\"\n\n    iex> conn() |> verify_session_payload(fn conn, _ -> conn end)\n    ** (RuntimeError) must be used after load_session/2","ref":"Charon.TokenPlugs.html#verify_session_payload/2-doctests","title":"Doctests - Charon.TokenPlugs.verify_session_payload/2","type":"function"},{"doc":"Generically verify that the bearer token payload contains `claim` and that its value matches `func`. The function must return the conn or an error message.\nMust be used after `verify_token_signature/2`.","ref":"Charon.TokenPlugs.html#verify_token_claim/2","title":"Charon.TokenPlugs.verify_token_claim/2","type":"function"},{"doc":"def verify_read_scope(conn, value) do\n      if \"read\" in String.split(value, \",\") do\n        conn\n      else\n        \"no read scope\"\n      end\n    end\n\n    iex> conn = conn() |> set_token_payload(%{\"scope\" => \"read,write\"})\n    iex> ^conn = conn |> verify_token_claim({\"scope\", &verify_read_scope/2})\n\n    # invalid\n    iex> conn = conn() |> set_token_payload(%{\"scope\" => \"write\"})\n    iex> conn |> verify_token_claim({\"scope\", &verify_read_scope/2}) |> Utils.get_auth_error()\n    \"no read scope\"\n\n    # claim must be present\n    iex> conn = conn() |> set_token_payload(%{})\n    iex> conn |> verify_token_claim({\"scope\", &verify_read_scope/2}) |> Utils.get_auth_error()\n    \"bearer token claim scope not found\"","ref":"Charon.TokenPlugs.html#verify_token_claim/2-doctests","title":"Doctests - Charon.TokenPlugs.verify_token_claim/2","type":"function"},{"doc":"Verify that the bearer token payload contains `claim` and that its value is `expected`.\nMust be used after `verify_token_signature/2`.","ref":"Charon.TokenPlugs.html#verify_token_claim_equals/2","title":"Charon.TokenPlugs.verify_token_claim_equals/2","type":"function"},{"doc":"iex> conn = conn() |> set_token_payload(%{\"type\" => \"access\"})\n    iex> ^conn = conn |> verify_token_claim_equals({\"type\", \"access\"})\n\n    # invalid\n    iex> conn = conn() |> set_token_payload(%{\"type\" => \"refresh\"})\n    iex> conn |> verify_token_claim_equals({\"type\", \"access\"}) |> Utils.get_auth_error()\n    \"bearer token claim type invalid\"\n\n    # claim must be present\n    iex> conn = conn() |> set_token_payload(%{})\n    iex> conn |> verify_token_claim_equals({\"type\", \"access\"}) |> Utils.get_auth_error()\n    \"bearer token claim type not found\"","ref":"Charon.TokenPlugs.html#verify_token_claim_equals/2-doctests","title":"Doctests - Charon.TokenPlugs.verify_token_claim_equals/2","type":"function"},{"doc":"Verify that the bearer token payload contains `claim` and that its value is in `expected`.\nMust be used after `verify_token_signature/2`.","ref":"Charon.TokenPlugs.html#verify_token_claim_in/2","title":"Charon.TokenPlugs.verify_token_claim_in/2","type":"function"},{"doc":"iex> conn = conn() |> set_token_payload(%{\"type\" => \"access\"})\n    iex> ^conn = conn |> verify_token_claim_in({\"type\", ~w(access)})\n\n    # invalid\n    iex> conn = conn() |> set_token_payload(%{\"type\" => \"refresh\"})\n    iex> conn |> verify_token_claim_in({\"type\", ~w(access)}) |> Utils.get_auth_error()\n    \"bearer token claim type invalid\"\n\n    # claim must be present\n    iex> conn = conn() |> set_token_payload(%{})\n    iex> conn |> verify_token_claim_in({\"type\", ~w(access)}) |> Utils.get_auth_error()\n    \"bearer token claim type not found\"","ref":"Charon.TokenPlugs.html#verify_token_claim_in/2-doctests","title":"Doctests - Charon.TokenPlugs.verify_token_claim_in/2","type":"function"},{"doc":"Verify that the bearer token payload contains a non-expired `exp` (expires at) claim.\nMust be used after `verify_token_signature/2`.\n\nNote that a token created by `Charon.SessionPlugs.upsert_session/3` is guaranteed\nto have an exp claim that does not outlive its underlying session.","ref":"Charon.TokenPlugs.html#verify_token_exp_claim/2","title":"Charon.TokenPlugs.verify_token_exp_claim/2","type":"function"},{"doc":"iex> conn = conn() |> set_token_payload(%{\"exp\" => Internal.now()})\n    iex> ^conn = conn |> verify_token_exp_claim([])\n\n    # some clock drift is allowed\n    iex> conn = conn() |> set_token_payload(%{\"exp\" => Internal.now() - 3})\n    iex> ^conn = conn |> verify_token_exp_claim([])\n\n    # expired\n    iex> conn = conn() |> set_token_payload(%{\"exp\" => Internal.now() - 6})\n    iex> conn |> verify_token_exp_claim([]) |> Utils.get_auth_error()\n    \"bearer token expired\"\n\n    # claim must be present\n    iex> conn = conn() |> set_token_payload(%{})\n    iex> conn |> verify_token_exp_claim([]) |> Utils.get_auth_error()\n    \"bearer token claim exp not found\"","ref":"Charon.TokenPlugs.html#verify_token_exp_claim/2-doctests","title":"Doctests - Charon.TokenPlugs.verify_token_exp_claim/2","type":"function"},{"doc":"Verify that the token (either access or refresh) is fresh.\n\nA token is fresh if it belongs to the current or previous \"refresh token generation\".\nA generation is a set of tokens that is created within a \"cycle TTL\"\namount of seconds from when the generation is first created.\nA new generation is created after the cycle TTL expires.\n\nSo a token must be \"fresh\", but because of refresh race conditions caused by\nnetwork issues or misbehaving clients, enforcing only a single fresh token causes too many problems in practice.\n\nMust be used after `load_session/2`. Verify the token type with `verify_token_claim_equals/2`.","ref":"Charon.TokenPlugs.html#verify_token_fresh/2","title":"Charon.TokenPlugs.verify_token_fresh/2","type":"function"},{"doc":"New cycle is created 5 seconds after the generation's first token is created,\nand token ttl is 24h (so irrelevant in this example).\n\n| When | New gen | Fresh tokens | Created token | Current gen (timestamp) | Previous gen | Comment                                                                            |\n|------|---------|--------------|---------------|-------------------------|--------------|------------------------------------------------------------------------------------|\n| 0    |         | -            | A             | A (0)                   | -            | Login                                                                              |\n| 10   | y       | A            | B             | B (10)                  | A            | Refresh after cycle TTL of A, [A] becomes prev gen                              |\n| 11   |         | A, B         | C             | B, C (10)               | A            | Refresh race within cycle TTL of B                                              |\n| 12   |         | A, B, C      | D             | B, C, D (10)            | A            | Refresh race within cycle TTL of B                                              |\n| 20   | y       | B, C, D      | E             | E (20)                  | B, C, D      | Refresh after cycle TTL of B, so [A] is now stale, and [B,C,D] becomes prev gen |\n| 30   | y       | E            | F             | F (30)                  | E            | Refresh after cycle TTL of E, so [B,C,D] is now stale, [E] becomes prev gen     |","ref":"Charon.TokenPlugs.html#verify_token_fresh/2-freshness-example","title":"Freshness example - Charon.TokenPlugs.verify_token_fresh/2","type":"function"},{"doc":"# some clock drift is allowed\n    iex> now = Internal.now()\n    iex> conn = conn() |> set_session(%{tokens_fresh_from: now, prev_tokens_fresh_from: now - 10}) |> set_token_payload(%{\"iat\" => now - 11})\n    iex> nil = conn |> verify_token_fresh(5) |> Utils.get_auth_error()\n\n    # if current gen is still within the cycle TTL, tokens from both it and previous gen are \"fresh\"\n    iex> now = Internal.now()\n    iex> conn = conn() |> set_session(%{tokens_fresh_from: now - 3, prev_tokens_fresh_from: now - 10})\n    iex> nil = conn |> set_token_payload(%{\"iat\" => now}) |> verify_token_fresh(5) |> Utils.get_auth_error()\n    # tokens are invalid from: iat   nil = conn |> set_token_payload(%{\"iat\" => now - 15}) |> verify_token_fresh(5) |> Utils.get_auth_error()\n    # older-than-previous-gen-and-clock-drift token is invalid\n    iex> conn |> set_token_payload(%{\"iat\" => now - 16}) |> verify_token_fresh(5) |> Utils.get_auth_error()\n    \"token stale\"\n    # no generation cycle will take place\n    iex> conn |> set_token_payload(%{\"iat\" => now}) |> verify_token_fresh(5) |> Internal.get_private(@cycle_token_generation)\n    false\n\n    # if current gen is too old, a generation cycle happens, and previous gen tokens are no longer valid\n    # tokens are invalid from: iat   now = Internal.now()\n    iex> conn = conn() |> set_session(%{tokens_fresh_from: now - 10, prev_tokens_fresh_from: now - 20})\n    iex> nil = conn |> set_token_payload(%{\"iat\" => now}) |> verify_token_fresh(5) |> Utils.get_auth_error()\n    # younger-than-current-gen-and-clock-drift token is valid\n    iex> nil = conn |> set_token_payload(%{\"iat\" => now - 15}) |> verify_token_fresh(5) |> Utils.get_auth_error()\n    # older-than-current-gen-and-clock-drift token is invalid\n    iex> conn |> set_token_payload(%{\"iat\" => now - 16}) |> verify_token_fresh(5) |> Utils.get_auth_error()\n    \"token stale\"\n    # a generation cycle will occur\n    iex> conn |> set_token_payload(%{\"iat\" => now}) |> verify_token_fresh(5) |> Internal.get_private(@cycle_token_generation)\n    true\n\n    # claim must be present\n    iex> conn = conn() |> set_session(%{tokens_fresh_from: 0, prev_tokens_fresh_from: 0}) |> set_token_payload(%{})\n    iex> conn |> verify_token_fresh(5) |> Utils.get_auth_error()\n    \"bearer token claim iat not found\"","ref":"Charon.TokenPlugs.html#verify_token_fresh/2-doctests","title":"Doctests - Charon.TokenPlugs.verify_token_fresh/2","type":"function"},{"doc":"Verify that the bearer token payload contains a valid `nbf` (not before) claim.\nMust be used after `verify_token_signature/2`.","ref":"Charon.TokenPlugs.html#verify_token_nbf_claim/2","title":"Charon.TokenPlugs.verify_token_nbf_claim/2","type":"function"},{"doc":"iex> conn = conn() |> set_token_payload(%{\"nbf\" => Internal.now()})\n    iex> ^conn = conn |> verify_token_nbf_claim([])\n\n    # some clock drift is allowed\n    iex> conn = conn() |> set_token_payload(%{\"nbf\" => Internal.now() + 3})\n    iex> ^conn = conn |> verify_token_nbf_claim([])\n\n    # not yet valid\n    iex> conn = conn() |> set_token_payload(%{\"nbf\" => Internal.now() + 6})\n    iex> conn |> verify_token_nbf_claim([]) |> Utils.get_auth_error()\n    \"bearer token not yet valid\"\n\n    # claim must be present\n    iex> conn = conn() |> set_token_payload(%{})\n    iex> conn |> verify_token_nbf_claim([]) |> Utils.get_auth_error()\n    \"bearer token claim nbf not found\"","ref":"Charon.TokenPlugs.html#verify_token_nbf_claim/2-doctests","title":"Doctests - Charon.TokenPlugs.verify_token_nbf_claim/2","type":"function"},{"doc":"Verify that the bearer token payload contains `claim`, *which is assumed to be an `:ordset`*,\nand that `ordset` (*which is also assumed to be either an ordset or a single element*)\n is a subset of that ordset.","ref":"Charon.TokenPlugs.html#verify_token_ordset_claim_contains/2","title":"Charon.TokenPlugs.verify_token_ordset_claim_contains/2","type":"function"},{"doc":"iex> conn = conn() |> set_token_payload(%{\"scope\" => ~w(a b c)})\n    iex> ^conn = conn |> verify_token_ordset_claim_contains({\"scope\", \"a\"})\n    iex> ^conn = conn |> verify_token_ordset_claim_contains({\"scope\", ~w(a b)})\n\n    # invalid\n    iex> conn = conn() |> set_token_payload(%{\"scope\" => ~w(a b c)})\n    iex> conn |> verify_token_ordset_claim_contains({\"scope\", \"d\"}) |> get_auth_error()\n    \"bearer token claim scope does not contain [d]\"\n    iex> conn |> verify_token_ordset_claim_contains({\"scope\", ~w(d e)}) |> get_auth_error()\n    \"bearer token claim scope does not contain [d, e]\"\n\n    # claim must be present\n    iex> conn = conn() |> set_token_payload(%{})\n    iex> conn |> verify_token_ordset_claim_contains({\"scope\", ~w(a b c)}) |> get_auth_error()\n    \"bearer token claim scope not found\"\n\n    # WATCH OUT!\n    # things will go horribly wrong if either the claim or the comparison value is not an ordset\n    iex> conn = conn() |> set_token_payload(%{\"scope\" => ~w(c b a)})\n    iex> conn |> verify_token_ordset_claim_contains({\"scope\", \"a\"}) |> get_auth_error()\n    \"bearer token claim scope does not contain [a]\"\n    iex> conn = conn() |> set_token_payload(%{\"scope\" => ~w(a b c)})\n    iex> conn |> verify_token_ordset_claim_contains({\"scope\", ~w(b a)}) |> get_auth_error()\n    \"bearer token claim scope does not contain [a]\"","ref":"Charon.TokenPlugs.html#verify_token_ordset_claim_contains/2-doctests","title":"Doctests - Charon.TokenPlugs.verify_token_ordset_claim_contains/2","type":"function"},{"doc":"Generically verify the bearer token payload.\nThe validation function `func` must return the conn or an error message.\nMust be used after `verify_token_signature/2`.","ref":"Charon.TokenPlugs.html#verify_token_payload/2","title":"Charon.TokenPlugs.verify_token_payload/2","type":"function"},{"doc":"iex> conn = conn() |> set_token_payload(%{})\n    iex> ^conn = conn |> verify_token_payload(fn conn, _pl -> conn end)\n\n    # invalid\n    iex> conn = conn() |> set_token_payload(%{\"scope\" => \"write\"})\n    iex> conn |> verify_token_payload(fn _conn, _pl -> \"no read scope\" end) |> Utils.get_auth_error()\n    \"no read scope\"\n\n    iex> conn() |> verify_token_payload(fn conn, _pl -> conn end)\n    ** (RuntimeError) must be used after verify_token_signature/2","ref":"Charon.TokenPlugs.html#verify_token_payload/2-doctests","title":"Doctests - Charon.TokenPlugs.verify_token_payload/2","type":"function"},{"doc":"Verify that the bearer token found by `get_token_from_auth_header/2` is signed correctly.","ref":"Charon.TokenPlugs.html#verify_token_signature/2","title":"Charon.TokenPlugs.verify_token_signature/2","type":"function"},{"doc":"iex> token = sign(%{\"msg\" => \"hurray!\"})\n    iex> conn = conn() |> set_token(token) |> verify_token_signature(@config)\n    iex> %{\"msg\" => \"hurray!\"} = Internal.get_private(conn, @bearer_token_payload)\n\n    # signature must match\n    iex> token = sign(%{\"msg\" => \"hurray!\"})\n    iex> conn = conn() |> set_token(token <> \"boom\") |> verify_token_signature(@config)\n    iex> Internal.get_private(conn, @bearer_token_payload)\n    nil\n    iex> Utils.get_auth_error(conn)\n    \"bearer token signature invalid\"\n\n    iex> conn() |> verify_token_signature(@config) |> Utils.get_auth_error()\n    \"bearer token not found\"","ref":"Charon.TokenPlugs.html#verify_token_signature/2-doctests","title":"Doctests - Charon.TokenPlugs.verify_token_signature/2","type":"function"},{"doc":"After verifying everything you would want to verify about a token,\nassign the following to the conn:\n- `:user_id` (bearer token claim \"sub\")\n- `:session_id` (bearer token claim \"sid\")\n- `:token_payload`\n- `:session` (if fetched with `Charon.TokenPlugs.load_session/2`)\n\nAll of the assign names are overridable:\n\n    # assign the user ID to key :current_user_id\n    plug PutAssigns, claims: %{\"sub\" => :current_user_id, \"sid\" => :session_id}","ref":"Charon.TokenPlugs.PutAssigns.html","title":"Charon.TokenPlugs.PutAssigns","type":"module"},{"doc":"iex> opts = PutAssigns.init([])\n    iex> conn = conn() |> set_token_payload(%{\"sub\" => 1, \"sid\" => \"a\"})\n    iex> conn |> PutAssigns.call(opts) |> Map.get(:assigns)\n    %{session_id: \"a\", token_payload: %{\"sid\" => \"a\", \"sub\" => 1}, user_id: 1}\n\n    iex> opts = PutAssigns.init(session: :da_session_baby)\n    iex> conn = conn() |> set_token_payload(%{\"sub\" => 1, \"sid\" => \"a\"}) |> set_session(\"hii\")\n    iex> conn |> PutAssigns.call(opts) |> Map.get(:assigns)\n    %{\n      session_id: \"a\",\n      token_payload: %{\"sid\" => \"a\", \"sub\" => 1},\n      user_id: 1,\n      da_session_baby: \"hii\"\n    }\n\n    # skipped on auth error\n    iex> opts = PutAssigns.init([])\n    iex> conn = conn() |> set_token_payload(%{\"sub\" => 1, \"sid\" => \"a\"}) |> set_auth_error(\"boom\")\n    iex> conn |> PutAssigns.call(opts) |> Map.get(:assigns)\n    %{}","ref":"Charon.TokenPlugs.PutAssigns.html#module-doctests","title":"Doctests - Charon.TokenPlugs.PutAssigns","type":"module"},{"doc":"Utility functions, mainly getters and setters for module internals.","ref":"Charon.Utils.html","title":"Charon.Utils","type":"module"},{"doc":"Get auth errors, if present.","ref":"Charon.Utils.html#get_auth_error/1","title":"Charon.Utils.get_auth_error/1","type":"function"},{"doc":"Get the bearer token, if present.","ref":"Charon.Utils.html#get_bearer_token/1","title":"Charon.Utils.get_bearer_token/1","type":"function"},{"doc":"Get the payload of the bearer token, if present.","ref":"Charon.Utils.html#get_bearer_token_payload/1","title":"Charon.Utils.get_bearer_token_payload/1","type":"function"},{"doc":"Get current session, if present.","ref":"Charon.Utils.html#get_session/1","title":"Charon.Utils.get_session/1","type":"function"},{"doc":"Get token signature transport mechanism, if present.","ref":"Charon.Utils.html#get_token_signature_transport/1","title":"Charon.Utils.get_token_signature_transport/1","type":"function"},{"doc":"Get token transport mechanism, if present.","ref":"Charon.Utils.html#get_token_transport/1","title":"Charon.Utils.get_token_transport/1","type":"function"},{"doc":"Get tokens, if present.","ref":"Charon.Utils.html#get_tokens/1","title":"Charon.Utils.get_tokens/1","type":"function"},{"doc":"Put an auth error on the conn","ref":"Charon.Utils.html#set_auth_error/2","title":"Charon.Utils.set_auth_error/2","type":"function"},{"doc":"Put a session on the conn","ref":"Charon.Utils.html#set_session/2","title":"Charon.Utils.set_session/2","type":"function"},{"doc":"Put a token on the conn","ref":"Charon.Utils.html#set_token/2","title":"Charon.Utils.set_token/2","type":"function"},{"doc":"Put a token payload on the conn","ref":"Charon.Utils.html#set_token_payload/2","title":"Charon.Utils.set_token_payload/2","type":"function"},{"doc":"Set token signature transport mechanism.\nMust be one of `\"bearer\"`, `\"cookie_only\"`, `\"cookie\"` `:bearer`, `:cookie_only` or `:cookie`.\n\nThis function only results in transports `:bearer` and `:cookie`, never in `:cookie_only`,\nin contrast with `set_token_signature_transport/2`, to maintain backwards compatibility.","ref":"Charon.Utils.html#set_token_signature_transport/2","title":"Charon.Utils.set_token_signature_transport/2","type":"function"},{"doc":"iex> :bearer = %Conn{} |> set_token_signature_transport(\"bearer\") |> get_token_signature_transport()\n    iex> :bearer = %Conn{} |> set_token_signature_transport(:bearer) |> get_token_signature_transport()\n    iex> :cookie = %Conn{} |> set_token_signature_transport(\"cookie_only\") |> get_token_signature_transport()\n    iex> :cookie = %Conn{} |> set_token_signature_transport(:cookie_only) |> get_token_signature_transport()\n    iex> :cookie = %Conn{} |> set_token_signature_transport(\"cookie\") |> get_token_signature_transport()\n    iex> :cookie = %Conn{} |> set_token_signature_transport(:cookie) |> get_token_signature_transport()","ref":"Charon.Utils.html#set_token_signature_transport/2-examples-doctests","title":"Examples / doctests - Charon.Utils.set_token_signature_transport/2","type":"function"},{"doc":"Set user id for session creation","ref":"Charon.Utils.html#set_user_id/2","title":"Charon.Utils.set_user_id/2","type":"function"},{"doc":"Derive a key from a base secret using PBKDF2.","ref":"Charon.Utils.KeyGenerator.html","title":"Charon.Utils.KeyGenerator","type":"module"},{"doc":"Derive a new key from `base_secret` using `salt`.\nThe result is cached using `:persistent_term` with key `Elixir.Charon.Utils.KeyGenerator`.","ref":"Charon.Utils.KeyGenerator.html#derive_key/3","title":"Charon.Utils.KeyGenerator.derive_key/3","type":"function"},{"doc":"- `:length` key length in bytes, default 32 (256 bits)\n  - `:iterations` hash iterations to derive new key, default 250_000\n  - `:digest` hashing algorithm used as pseudo-random function, default `:sha256`","ref":"Charon.Utils.KeyGenerator.html#derive_key/3-options","title":"Options - Charon.Utils.KeyGenerator.derive_key/3","type":"function"},{"doc":"iex> derive_key(\"secret\", \"salt\", length: 5, iterations: 1)\n    <<56, 223, 66, 139, 48>>\n\n    # key is returned from cache based on function args\n    iex> :persistent_term.put(KeyGenerator, %{{\"secret\", \"salt\", [length: 5, iterations: 1]} => \"supersecret\"})\n    iex> derive_key(\"secret\", \"salt\", length: 5, iterations: 1)\n    \"supersecret\"","ref":"Charon.Utils.KeyGenerator.html#derive_key/3-doctests","title":"Doctests - Charon.Utils.KeyGenerator.derive_key/3","type":"function"},{"doc":"","ref":"Charon.Utils.KeyGenerator.html#t:opts/0","title":"Charon.Utils.KeyGenerator.opts/0","type":"type"},{"doc":"Error raised when token transport method \"bearer\" is requested by a browser client.","ref":"Charon.SessionPlugs.InsecureTokenTransportError.html","title":"Charon.SessionPlugs.InsecureTokenTransportError","type":"exception"},{"doc":"Error raised when a new/updated session could not be stored.","ref":"Charon.SessionPlugs.SessionStorageError.html","title":"Charon.SessionPlugs.SessionStorageError","type":"exception"},{"doc":"Error raised on an optimistic locking error when updating an existing session.","ref":"Charon.SessionPlugs.SessionUpdateConflictError.html","title":"Charon.SessionPlugs.SessionUpdateConflictError","type":"exception"},{"doc":"# Charon\n\nCharon is an extensible auth framework for Elixir, mostly for API's. The base package provides token generation & verification, and session handling. Additional functionality can be added with child packages. The package is opinionated with sane config defaults as much as possible.","ref":"readme.html","title":"Charon","type":"extras"},{"doc":"<!-- TOC -->\n\n- [Charon](#Charon)\n  - [Table of contents](#table-of-contents)\n  - [Features](#features)\n  - [Child packages](#child-packages)\n  - [Documentation](#documentation)\n  - [How to use](#how-to-use)\n    - [Installation](#installation)\n    - [Configuration](#configuration)\n    - [Setting up a session store](#setting-up-a-session-store)\n    - [Protecting routes](#protecting-routes)\n    - [Logging in, logging out and refreshing](#logging-in-logging-out-and-refreshing)\n    - [Testing](#testing)\n  - [Copyright and License](#copyright-and-license)\n\n<!-- /TOC -->","ref":"readme.html#table-of-contents","title":"Table of contents - Charon","type":"extras"},{"doc":"- Striking a \"statefulness\" balance is left to the application. By default, access token are fully stateless, and refresh tokens are backed by a server-side session.\n- A single base secret is used to derive special-purpose keys (like Phoenix's `:secret_key_base`), simplifying key management\n- Tokens are generated by a factory behaviour with a default implementation using symmetric-key signed JWT's.\n- Default token factory supports key rotation (including new derived keys)\n- Default token factory supports asymmetric signatures using EdDSA with Ed25519 or Ed448 curves.\n- Token verification is completely customizable using simple helper functions.\n- Sessions are managed by a session store behaviour with a default implementation using Redis.\n- To support both web- and other clients, it is possible to \"split-off\" the token's signatures and put them in cookies (to prevent storing tokens in LocalStorage and other insecure storage).\n  This idea is inspired by [this](https://medium.com/lightrail/getting-token-authentication-right-in-a-stateless-single-page-application-57d0c6474e3) excellent Medium article by Peter Locke.\n- Flexible configuration that does not (have to) depend on the application environment.\n- Small number of dependencies.","ref":"readme.html#features","title":"Features - Charon","type":"extras"},{"doc":"- [CharonAbsinthe](https://github.com/weareyipyip/charon_absinthe) for using Charon with Absinthe.\n- [CharonOauth2](https://github.com/weareyipyip/charon_oauth2) adds Oauth2 authorization server capability.\n- [CharonLogin](https://github.com/weareyipyip/charon_login) adds authentication flows/challenges to generically provide login checks like password and MFA.\n- (planned) CharonSocial for social login.\n- (planned) CharonPermissions for authorization checks.","ref":"readme.html#child-packages","title":"Child packages - Charon","type":"extras"},{"doc":"Documentation can be found at [https://hexdocs.pm/charon](https://hexdocs.pm/charon).","ref":"readme.html#documentation","title":"Documentation - Charon","type":"extras"},{"doc":"","ref":"readme.html#how-to-use","title":"How to use - Charon","type":"extras"},{"doc":"The package can be installed by adding `charon` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:charon, \"~> 3.0\"}\n  ]\nend\n```","ref":"readme.html#installation","title":"Installation - Charon","type":"extras"},{"doc":"Configuration has been made easy using a config helper struct `Charon.Config`, which has a function `from_enum/1` that verifies that your config is complete and valid, raising on missing fields. By using multiple config structs, you can support multiple configurations within a single application. The main reason to use multiple sets of configuration is that you can support different auth requirements in this way. You could, for example, create a never-expiring session for ordinary users and create a short-lived session for application admins.\n\n```elixir\n# Charon itself only requires a token issuer and a base secret getter.\n@my_config Charon.Config.from_enum(\n             token_issuer: \"MyApp\",\n             get_base_secret: &MyApp.get_base_secret/0\n           )\n\n# it is possible to use the application environment as well if you wish\n@my_config Application.compile_env(:my_app, :charon) |> Charon.Config.from_enum()\n```","ref":"readme.html#configuration","title":"Configuration - Charon","type":"extras"},{"doc":"A session store can be created using multiple state stores, be it a database or a GenServer. All you have to do is implement a simple behaviour which you can find in `Charon.SessionStore.Behaviour`. Two default implementations are provided, `Charon.SessionStore.RedisStore` uses a Redis database, `Charon.SessionStore.LocalStore` uses a GenServer. Use `Charon.SessionStore.DummyStore` in case you don't want to use server-side sessions and prefer fully stateless tokens. The default and recommended option is RedisStore. Use LocalStore for local testing only - it is NOT persistent.\n\nIn order to use RedisStore, add `Charon.SessionStore.RedisStore` to your supervision tree:\n\n```elixir\n# application.ex\n\ndef start(_, ) do\n  redix_opts = [host: \"localhost\", port: 6379, password: \"supersecret\", database: 0]\n\n  children = [\n    ...\n    {Charon.SessionStore.RedisStore, pool_size: 15, redix_opts: redix_opts},\n    ...\n  ]\n\n  opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n  Supervisor.start_link(children, opts)\nend\n```","ref":"readme.html#setting-up-a-session-store","title":"Setting up a session store - Charon","type":"extras"},{"doc":"The default token factory is `Charon.TokenFactory.Jwt`, which requires no configuration by default, and uses symmetric HMAC-SHA256 signatures and a derived key. You can override its settings or set up your own TokenFactory by implementing `Charon.TokenFactory.Behaviour`.","ref":"readme.html#setting-up-a-token-factory","title":"Setting up a token factory - Charon","type":"extras"},{"doc":"Verifying incoming tokens is supported by the plugs in `Charon.TokenPlugs` (and submodules).\nYou can use these plugs to create pipelines to verify the tokens and their claims.\nNote that the plugs don't halt the connection until you call `Charon.TokenPlugs.verify_no_auth_error/2`, but further processing stops as soon as a previous plug adds an error to the conn.\nExample access- and refresh token pipelines:\n\n```elixir\ndefmodule MyApp.AccessTokenPipeline do\n  @moduledoc \"\"\"\n  Verify access tokens. A access token:\n   - must have a valid signature\n   - must not be expired (and already valid)\n   - must have a \"type\" claim with value \"access\"\n  \"\"\"\n  use Plug.Builder\n  import Charon.TokenPlugs\n\n  @config Application.compile_env(:my_app, :charon) |> Charon.Config.from_enum()\n\n  plug :get_token_from_auth_header\n  plug :get_token_from_cookie, @config.access_cookie_name\n  plug :verify_token_signature, @config\n  plug :verify_token_nbf_claim\n  plug :verify_token_exp_claim\n  plug :verify_token_claim_equals, {\"type\", \"access\"}\n  plug :verify_no_auth_error, &MyApp.TokenErrorHandler.on_error/2\n  plug Charon.TokenPlugs.PutAssigns\nend\n\ndefmodule MyApp.RefreshTokenPipeline do\n  @moduledoc \"\"\"\n  Verify refresh tokens. A refresh token:\n   - must have a valid signature\n   - must not be expired (and already valid)\n   - must have a \"type\" claim with value \"refresh\"\n   - must have a corresponding session\n   - must be fresh (see `Charon.TokenPlugs.verify_token_fresh/2` docs)\n  \"\"\"\n  use Plug.Builder\n  import Charon.TokenPlugs\n\n  @config Application.compile_env(:my_app, :charon) |> Charon.Config.from_enum()\n\n  plug :get_token_from_auth_header\n  plug :get_token_from_cookie, @config.refresh_cookie_name\n  plug :verify_token_signature, @config\n  plug :verify_token_nbf_claim\n  plug :verify_token_exp_claim\n  plug :verify_token_claim_equals, {\"type\", \"refresh\"}\n  plug :load_session, @config\n  plug :verify_token_fresh, 10\n  plug :verify_no_auth_error, &MyApp.TokenErrorHandler.on_error/2\n  plug Charon.TokenPlugs.PutAssigns\nend\n\n# use the pipelines in your router\ndefmodule MyAppWeb.Router do\n  use MyAppWeb, :router\n\n  pipeline :api do\n    plug :accepts, [\"json\"]\n  end\n\n  pipeline :valid_access_token do\n    # or just use the token plugs right here instead of putting them in a separate module\n    plug MyApp.AccessTokenPipeline\n  end\n\n  pipeline :valid_refresh_token do\n    plug MyApp.RefreshTokenPipeline\n  end\n\n  scope \"/\" do\n    pipe_through [:api]\n\n    post \"/current_session\", SessionController, :login\n  end\n\n  scope \"/\" do\n    pipe_through [:api ,:valid_access_token]\n\n    delete \"/current_session\", SessionController, :logout\n  end\n\n  scope \"/\" do\n    pipe_through [:api ,:valid_refresh_token]\n\n    # optionally limit the refresh cookie path to this path using `Config.refresh_cookie_opts`\n    post \"/current_session/refresh\", SessionController, :refresh\n  end\nend\n```","ref":"readme.html#protecting-routes","title":"Protecting routes - Charon","type":"extras"},{"doc":"Create a session controller with login, logout and refresh routes. You can use `Charon.SessionPlugs` for all operations.\n\n```elixir\ndefmodule MyAppWeb.SessionController do\n  @moduledoc \"\"\"\n  Controller for a user's session(s), including login, logout and refresh.\n  \"\"\"\n  use MyAppWeb, :controller\n\n  alias Charon.{SessionPlugs, Utils}\n  alias MyApp.{User, Users}\n\n  @config Application.compile_env(:my_app, :charon) |> Charon.Config.from_enum()\n\n  @doc \"\"\"\n  Login to my app!\n  \"\"\"\n  def login(conn, %{\n        \"email\" => email,\n        \"password\" => password,\n        \"token_transport\" => token_transport\n      })\n      when token_transport in ~w(bearer cookie_only cookie) do\n    with {:ok, user} <- Users.get_by(email: email) |> Users.verify_password(password) do\n      # you can do extra checks here, like checking if the user is active, for example\n\n      conn\n      # you can add/override claims in the tokens (be careful!)\n      |> SessionPlugs.upsert_session(\n        @config,\n        user_id: user.id,\n        token_transport: token_transport,\n        access_claim_overrides: %{\"roles\" => user.roles}\n      )\n      |> put_status(201)\n      |> send_token_response(user)\n    else\n      _error -> send_resp(conn, 401, \"user not found or wrong password\")\n    end\n  end\n\n  @doc \"\"\"\n  Logout from my app!\n  \"\"\"\n  def logout(conn, _params) do\n    conn\n    |> SessionPlugs.delete_session(@config)\n    |> send_resp(204, \"\")\n  end\n\n  @doc \"\"\"\n  Stay fresh with my app!\n  \"\"\"\n  def refresh(%{assigns: %{current_user_id: user_id}} = conn, _params) do\n    with %User{status: \"active\"} = user <- Users.get_by(id: user_id) do\n      # here you can do extra checks again\n\n      conn\n      # there's no need to set user_id, token transport or extra session payload\n      # because these are grabbed from the current session\n      # but all added/overridden token claims must be passed in again\n      |> SessionPlugs.upsert_session(@config, access_claim_overrides: %{\"roles\" => user.roles})\n      |> send_token_response(user)\n    else\n      _error -> send_resp(conn, 401, \"user not found or inactive\")\n    end\n  end\n\n  ###########\n  # Private #\n  ###########\n\n  defp send_token_response(conn, user) do\n    session = conn |> Utils.get_session() |> Map.from_struct()\n    tokens = conn |> Utils.get_tokens() |> Map.from_struct()\n    json(conn, %{tokens: tokens, session: session})\n  end\nend\n```\n\nAnd that's it :) Optionally, you can add get-all, logout-all and logout-other session endpoints, if your session store supports it (the default Redis one does).","ref":"readme.html#logging-in-logging-out-and-refreshing","title":"Logging in, logging out and refreshing - Charon","type":"extras"},{"doc":"To aid in testing, some utility functions have been added for your convenience.\n\n```elixir\ndefmodule MyAppWeb.ControllerTest do\n  use MyAppWeb.ConnCase\n  alias Charon.TestHelpers\n  alias MyApp.User\n\n  setup seeds do\n    # create a user somehow...\n    user = %User{id: 1}\n\n    # create a valid session with tokens and put the access token on the conn\n    conn = TestHelpers.put_token(conn, user.id, MyAppWeb.Charon.config())\n\n    [conn: conn]\n  end\n\nend\n```","ref":"readme.html#testing","title":"Testing - Charon","type":"extras"},{"doc":"Copyright (c) 2023, YipYip B.V.\n\nCharon source code is licensed under the [Apache-2.0 License](./LICENSE.md)","ref":"readme.html#copyright-and-license","title":"Copyright and License - Charon","type":"extras"},{"doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS","ref":"license.html","title":"LICENSE","type":"extras"},{"doc":"# Upgrade guide","ref":"upgrade_guide.html","title":"Upgrade guide","type":"extras"},{"doc":"","ref":"upgrade_guide.html#2-x-to-3-x","title":"2.x to 3.x - Upgrade guide","type":"extras"},{"doc":"If you use `Charon.SessionStore.RedisStore` you must do the following before upgrading to 3.x:\n\n1.  Update Charon to >= 2.8.0\n1.  Upgrade your Redis instance to 7.x.x (this can be an in-place upgrade)\n1.  Run `Charon.SessionStore.RedisStore.migrate_sessions/1`\n1.  If you explicitly set config options `:allow_unsigned?` and `:redix_module`, you must remove your overrides. Support for these config options has been dropped.\n1.  Initialize the Redix connection pool in your supervision tree. See readme section [setting up a sessionstore](./README.md#setting-up-a-session-store).\n1.  Remove your Redix module (if you don't use it for anything else).","ref":"upgrade_guide.html#redisstore-changes","title":"RedisStore changes - Upgrade guide","type":"extras"},{"doc":"All marked-as-deprecated functions have been dropped, so make sure your code does not use these functions anymore. The compiler emits warnings if you do.","ref":"upgrade_guide.html#deprecated-functions","title":"Deprecated functions - Upgrade guide","type":"extras"},{"doc":"# Changelog","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"","ref":"changelog.html#3-x","title":"3.x - Changelog","type":"extras"},{"doc":"- `Charon.SessionStore.RedisStore`\n\n  - requires Redis >= 7.x.x\n  - uses a Redix connection pool by itself, which requires initialization under the application supervision tree\n  - implements optimistic locking\n  - uses a new storage format based on hash sets, to which sessions are migrated starting from Charon 2.8\n  - uses Redis functions to implement all session store operations in a single round trip to the Redis instance\n  - support for unsigned binaries has been dropped\n    - config options `:allow_unsigned?` has been removed\n    - `migrate_sessions/1` has been removed\n    - sessions that have not been migrated using `migrate_sessions/1` can no longer be used\n\n- `Charon.SessionStore.LocalStore`\n\n  - implements optimistic locking\n\n- 2.x marked-deprecated functions have been removed:\n\n  - `Charon.Models.Session.deserialize/2`\n  - `Charon.Models.Session.serialize/1`\n  - `Charon.SessionStore.delete/3`\n  - `Charon.SessionStore.get/3`\n  - `Charon.SessionStore.delete_all/2`\n  - `Charon.SessionStore.get_all/2`\n  - `Charon.SessionStore.RedisStore.cleanup/1`\n  - `Charon.TokenPlugs.verify_refresh_token_fresh/2`\n\n- `Charon.TokenPlugs.verify_token_signature/2` no longer adds default value \"full\" for claim \"styp\".\n  This should not result in issues for tokens created by Charon 2.x.x deployments.\n  Older deployments may wish to add a plug after `Charon.TokenPlugs.verify_token_signature/2` that adds this default claim.","ref":"changelog.html#breaking","title":"Breaking - Changelog","type":"extras"}],"producer":{"name":"ex_doc","version":"0.35.1"},"proglang":"elixir"}